@@ -1,4 +1,4 @@
-/* tftp-hpa: $Id: tftpd.c,v 1.54 2003/04/12 06:54:58 hpa Exp $ */
+/* tftp-hpa: $Id: tftpd.c,v 1.56 2004/01/08 20:48:30 hpa Exp $ */
 
 /* $OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $	*/
 
@@ -45,7 +45,7 @@ static const char *copyright UNUSED =
 /*static char sccsid[] = "from: @(#)tftpd.c	5.13 (Berkeley) 2/26/91";*/
 /*static char rcsid[] = "$OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $: tftpd.c,v 1.6 1997/02/16 23:49:21 deraadt Exp $";*/
 static const char *rcsid UNUSED =
-"tftp-hpa $Id: tftpd.c,v 1.54 2003/04/12 06:54:58 hpa Exp $";
+"tftp-hpa $Id: tftpd.c,v 1.56 2004/01/08 20:48:30 hpa Exp $";
 #endif /* not lint */
 
 /*
@@ -92,8 +92,10 @@ int           timeout_quit = 0;
 sigjmp_buf    timeoutbuf;
 
 #define	PKTSIZE	MAX_SEGSIZE+4
-char	buf[PKTSIZE];
-char	ackbuf[PKTSIZE];
+char		buf[PKTSIZE];
+char		ackbuf[PKTSIZE];
+unsigned int	max_blksize = MAX_SEGSIZE;
+
 struct	sockaddr_in from;
 int	fromlen;
 off_t	tsize;
@@ -279,7 +281,7 @@ main(int argc, char **argv)
   
   openlog(__progname, LOG_PID|LOG_NDELAY, LOG_DAEMON);
   
-  while ((c = getopt(argc, argv, "cspvVla:u:U:r:t:T:m:")) != -1)
+  while ((c = getopt(argc, argv, "cspvVla:B:u:U:r:t:T:m:")) != -1)
     switch (c) {
     case 'c':
       cancreate = 1;
@@ -299,6 +301,17 @@ main(int argc, char **argv)
     case 't':
       waittime = atoi(optarg);
       break;
+    case 'B':
+      {
+	char *vp;
+	max_blksize = (unsigned int)strtoul(optarg, &vp, 10);
+	if ( max_blksize < 512 || max_blksize > MAX_SEGSIZE || *vp ) {
+	  syslog(LOG_ERR, "Bad maximum blocksize value (range 512-%d): %s",
+		 MAX_SEGSIZE, optarg);
+	  exit(EX_USAGE);
+	}
+      }
+      break;
     case 'T':
       {
 	char *vp;
@@ -474,9 +487,7 @@ main(int argc, char **argv)
 	close(0); close(1); close(2);
       }
 #ifdef HAVE_SETSID
-#ifndef __CYGWIN__		/* Kills the process on Cygwin? */
       setsid();
-#endif
 #endif
     }
   } else {
@@ -817,8 +828,8 @@ set_blksize(char *val, char **ret)
   
   if (sz < 8)
     return(0);
-  else if (sz > MAX_SEGSIZE)
-    sz = MAX_SEGSIZE;
+  else if (sz > max_blksize)
+    sz = max_blksize;
   
   segsize = sz;
   sprintf(*ret = b_ret, "%u", sz);
@@ -845,8 +856,8 @@ set_blksize2(char *val, char **ret)
   
   if (sz < 8)
     return(0);
-  else if (sz > MAX_SEGSIZE)
-    sz = MAX_SEGSIZE;
+  else if (sz > max_blksize)
+    sz = max_blksize;
   
   /* Convert to a power of two */
   if ( sz & (sz-1) ) {
